#ifndef __JABYENGINE_FILE_TYPES_HPP__
#define __JABYENGINE_FILE_TYPES_HPP__
#include "../Auxiliary/bits.hpp"
#include "../jabyengine_defines.h"

namespace JabyEngine {
    struct __no_align Nothing {
    };

    struct __no_align SimpleTIM {
        static constexpr auto TextureX = BitRange::from_to(0, 8);
        static constexpr auto TextureY = BitRange::from_to(9, 16);
        static constexpr auto ClutX    = BitRange::from_to(17, 22);
        static constexpr auto ClutY    = BitRange::from_to(23, 31);

        uint32_t raw;

        constexpr SimpleTIM() {
            this->raw = 0;
        }

        constexpr SimpleTIM(uint16_t texX, uint16_t texY, uint16_t clutX, uint16_t clutY) : raw(TextureX.as_value(texX >> 1) | TextureY.as_value(texY >> 1) | ClutX.as_value(clutX >> 4) | ClutY.as_value(clutY)) {
        }

        constexpr uint16_t getTextureX() const {
            return bit::value::get_normalized(this->raw, TextureX) << 1;
        }

        constexpr uint16_t getTextureY() const {
            return bit::value::get_normalized(this->raw, TextureY) << 1;
        }

        constexpr uint16_t getClutX() const {
            return bit::value::get_normalized(this->raw, SimpleTIM::ClutX) << 4;
        }

        constexpr uint16_t getClutY() const {
            return bit::value::get_normalized(this->raw, ClutY);
        }
    };

    struct __no_align CopyTo {
        uint32_t* dst;
    };

    typedef CopyTo Overlay;
}
#endif // !__JABYENGINE_FILE_TYPES_HPP__