#include "../../internal-include/BootLoader/boot_loader.hpp"
#include <PSX/System/IOPorts/dMa_io.hpp>

// 2x For setup timing
#include <PSX/Timer/high_res_timer.hpp>
#include <stdio.h>

namespace JabyEngine {
    namespace boot {
        namespace Start {
            //This should become part of the bootloader later
            static void enable_DMA() {
                DMA_IO::DPCR = DMA_IO::DPCR_t(DMA_IO::DPCR).set(DMA_IO::DPCR_t::SPUEnable).set(DMA_IO::DPCR_t::GPUEnable).set(DMA_IO::DPCR_t::CDROMEnable);
            }

            static void setup() {
                enable_DMA();

                SPU::stop_voices();
                
                CD::setup();
                Timer::setup();

                const auto start = HighResTime::get_time_stamp();
                printf("Start...\n");
                GPU::setup();
                GPU::display_logo();
                const auto end   = HighResTime::get_time_stamp();
                printf("GPU setup took %ims %ius\n", start.milliseconds_to(end), start.microseconds_to(end));
                
                //Pause??

                SPU::setup();
            }
        }
    }

    void start() {
        printf("Starting Planschbecken\n");
        boot::Start::setup();
        
        printf("Running main...\n");
        run();
    }
}