pub struct BitRange {
    start: usize,
    length: usize
}

impl BitRange {
    pub const fn from_to(start_bit: usize, end_bit: usize) -> Self {
        Self{start: start_bit, length: (end_bit - start_bit) + 1}
    }

    pub const fn get_mask(&self) -> usize {
        self.max_value()
    }

    pub const fn max_value(&self) -> usize {
        (1 << self.length) - 1
    }

    pub const fn or_value(&self, dst_value: usize, value: usize) -> usize {
        dst_value | ((value & self.get_mask()) << self.start)
    }
}

pub struct Bit {
    pos: usize
}

impl Bit {
    pub const fn at(pos: usize) -> Self {
        Bit{pos}
    }

    pub const fn or_value(&self, dst_value: usize, is_set: bool) -> usize {
        if is_set {
            dst_value | (1 << self.pos)
        }

        else {
            dst_value
        }
    }
}